//===========================================================================
//  $Name: arts++-1-1-a12 $
//  $Id: ArtsBgp4AsPathAttribute.hh,v 1.2 2004/04/21 23:51:25 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4ASPATHATTRIBUTE_HH_
#define _ARTSBGP4ASPATHATTRIBUTE_HH_

extern "C" {
#include "caida_t.h"
}

#include "ArtsBgp4AsPathSegment.hh"

//---------------------------------------------------------------------------
//  class ArtsBgp4AsPathAttribute
//---------------------------------------------------------------------------
//!  This class abstracts a BGP4 AS path attribute.  It contains a
//!  vector of ArtsBgp4AsPathSegment objects.
//---------------------------------------------------------------------------
class ArtsBgp4AsPathAttribute
{
public:
  //--------------------------------------------------------------------------
  //                        ArtsBgp4AsPathAttribute()
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsBgp4AsPathAttribute();
  
  //--------------------------------------------------------------------------
  //                       ~ArtsBgp4AsPathAttribute()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsBgp4AsPathAttribute();

  //-------------------------------------------------------------------------
  //                      uint8_t  NumSegments() const
  //.........................................................................
  //!  Returns the number of ArtsBgp4AsPathSegment objects contained
  //!  in the ArtsBgp4AsPathAttribute.
  //-------------------------------------------------------------------------
  uint8_t  NumSegments() const;
  
  //-------------------------------------------------------------------------
  //                uint8_t  NumSegments(uint8_t numSegments)
  //.........................................................................
  //!  Sets and returns the number of ArtsBgp4AsPathSegment objects
  //!  contained in the ArtsBgp4AsPathAttribute.
  //-------------------------------------------------------------------------
  uint8_t  NumSegments(uint8_t numSegments);
  
  //-------------------------------------------------------------------------
  //            std::vector<ArtsBgp4AsPathSegment> & Segments() const
  //.........................................................................
  //!  Returns a reference to the vector of ArtsBgp4AsPathSegment objects
  //!  in the ArtsBgp4AsPathAttribute.  Note that we can call this member
  //!  on a const object even though we don't return a const reference;
  //!  the vector of ArtsBgp4AsPathSegment objects is mutable.
  //-------------------------------------------------------------------------
  std::vector<ArtsBgp4AsPathSegment> & Segments() const;

  //-------------------------------------------------------------------------
  //           operator = (const ArtsBgp4AsPathAttribute & asPath)
  //.........................................................................
  //!  Overloaded '=' operator for assigning the value of one
  //!  ArtsBgp4AsPathAttribute to another.
  //-------------------------------------------------------------------------
  ArtsBgp4AsPathAttribute &
  operator = (const ArtsBgp4AsPathAttribute & asPath);

  //--------------------------------------------------------------------------
  //            std::istream & read(std::istream & is, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4AsPathAttribute from an istream.
  //!  Returns the istream.
  //--------------------------------------------------------------------------
  std::istream & read(std::istream & is, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //..........................................................................
  //!  Reads the ocntents of an ArtsBgp4AsPathAttribute from a file
  //!  descriptor.  Returns the number of bytes read on success, -1 on
  //!  failure.  
  //--------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //        std::ostream & write(std::ostream & os, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4AsPathAttribute to an ostream.
  //!  Returns the ostream.
  //--------------------------------------------------------------------------
  std::ostream & write(std::ostream & os, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const;              
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4AsPathAttribute to a file
  //!  descriptor.  Returns the number of bytes written on success, -1 on
  //!  failure.
  //--------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //                              void Unique()
  //..........................................................................
  //
  //--------------------------------------------------------------------------
  void Unique();

  //--------------------------------------------------------------------------
  //               uint32_t Length(uint8_t version = 0) const
  //..........................................................................
  //!  Returns the number of bytes required to store the
  //!  ArtsBgp4AsPathAttribute on disk.
  //--------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //       friend std::ostream & 
  //       operator << (std::ostream & os, 
  //                const ArtsBgp4AsPathAttribute & bgp4AsPathAttribute) 
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4AsPathAttribute to an ostream in
  //!  a user-friendly form.
  //--------------------------------------------------------------------------
  friend std::ostream &
  operator << (std::ostream & os,
               const ArtsBgp4AsPathAttribute & bgp4AsPathAttribute);
  
  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG
  
private:
  mutable std::vector<ArtsBgp4AsPathSegment>   _segments;

  static uint32_t  _numObjects;
};

#endif  // _ARTSBGP4ASPATHATTRIBUTE_HH_
