//===========================================================================
//  $Name: arts++-1-1-a12 $
//  $Id: ArtsBgp4RouteEntry.hh,v 1.2 2004/04/21 23:51:25 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4ROUTEENTRY_HH_
#define _ARTSBGP4ROUTEENTRY_HH_

extern "C" {
#include "caida_t.h"
}

#include "ArtsBgp4Attribute.hh"

//---------------------------------------------------------------------------
//  class ArtsBgp4RouteEntry
//---------------------------------------------------------------------------
//!  This class encapsualtes a route entry from BGP4.
//---------------------------------------------------------------------------
class ArtsBgp4RouteEntry
{
public:
  //--------------------------------------------------------------------------
  //                          ArtsBgp4RouteEntry()
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsBgp4RouteEntry();

  //--------------------------------------------------------------------------
  //!  constructor
  //..........................................................................
  //!  This is here just to make Ipv4PrefixPatricia happy.
  //--------------------------------------------------------------------------
  ArtsBgp4RouteEntry(int);
  
  //--------------------------------------------------------------------------
  //                          ~ArtsBgp4RouteEntry()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsBgp4RouteEntry();

  //-------------------------------------------------------------------------
  //                     uint32_t AttributeIndex() const
  //.........................................................................
  //!  Returns a bitfield indicating which attributes are present for the
  //!  route entry.
  //-------------------------------------------------------------------------
  uint32_t AttributeIndex() const;

  const ArtsBgp4AsPathAttribute * AsPathAttribute() const;
    
  //-------------------------------------------------------------------------
  //               uint32_t AttributeIndex(uint32_t attrIndex)
  //.........................................................................
  //!  Sets and returns a bitfield indicating which attributes are
  //!  present for the route entry.
  //-------------------------------------------------------------------------
  uint32_t AttributeIndex(uint32_t attrIndex);

  //--------------------------------------------------------------------------
  //            std::istream & read(std::istream& is, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4RouteEntry from an istream.
  //!  Returns the istream.
  //--------------------------------------------------------------------------
  std::istream & read(std::istream& is, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //..........................................................................
  //!  Reads the contents of an ArtsBgp4RouteEntry from a file descriptor.
  //!  Returns the number of bytes read on success, -1 on failure.
  //--------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);

  //--------------------------------------------------------------------------
  //        std::ostream & write(std::ostream & os, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4RouteEntry to an ostream.
  //!  Returns the ostream.
  //--------------------------------------------------------------------------
  std::ostream & write(std::ostream & os, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //..........................................................................
  //!  Writes the contents of an ArtsBgp4RouteEntry to a file descriptor.
  //!  Returns the number of bytes written on success, -1 on failure.
  //--------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //               uint32_t Length(uint8_t version = 0) const;               
  //..........................................................................
  //!  Returns the number of bytes required to store the ArtsBgp4RouteEntry
  //!  on disk.
  //--------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //--------------------------------------------------------------------------
  //        bool AddBgp4Attribute(const ArtsBgp4Attribute & bgp4Attr)
  //..........................................................................
  //!  If we already have an attribute of the same type as \c bgpAttr, does
  //!  nothing and returns false.  Else adds the \c bgp4Attr attribute to
  //!  the route entry.
  //--------------------------------------------------------------------------
  bool AddBgp4Attribute(const ArtsBgp4Attribute & bgp4Attr);

  //--------------------------------------------------------------------------
  //       std::vector<ArtsBgp4Attribute> & BgpAttributes() const 
  //..........................................................................
  //!  Returns a reference to the vector of attributes for the route
  //!  entry.
  //--------------------------------------------------------------------------
  std::vector<ArtsBgp4Attribute> & BgpAttributes() const
  {
    return(this->_attributes);
  }

  //--------------------------------------------------------------------------
  //                        void ClearBgpAttributes()
  //..........................................................................
  //!  Removes all attributes from the route entry.
  //--------------------------------------------------------------------------
  void ClearBgpAttributes();

  //--------------------------------------------------------------------------
  //          ArtsBgp4RouteEntry &                          
  //          operator = (const ArtsBgp4RouteEntry & bgp4RouteEntry)
  //..........................................................................
  //!  Deep copies an ArtsBgp4RouteEntry.
  //--------------------------------------------------------------------------
  ArtsBgp4RouteEntry &
  operator = (const ArtsBgp4RouteEntry & bgp4RouteEntry);

  //--------------------------------------------------------------------------
  // friend std::ostream & operator << (std::ostream & os,               
  //                               const ArtsBgp4RouteEntry & bgp4RouteEntry)
  //..........................................................................
  //!  Writes an ArtsBgp4RouteEntry to an ostream in a human-friendly form.
  //--------------------------------------------------------------------------
  friend std::ostream & operator << (std::ostream & os,
                                const ArtsBgp4RouteEntry & bgp4RouteEntry);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                  static uint32_t NumObjects()
    //........................................................................
    //
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint32_t                           _attrIndex;
  mutable std::vector<ArtsBgp4Attribute>  _attributes;

  static uint32_t  _numObjects;
};

#endif  // _ARTSBGP4ROUTEENTRY_HH_
